// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos.Basics
{
    // 単一の TextLayout を使用して大規模な PDF を生成します。
    // 他の大規模なドキュメントのサンプル（StartEndDoc、LargeDocument2）とは異なり、
    // このサンプルは実際には段落の単一のシーケンスとしてドキュメント全体をページ化します。
    // 各ページに少なくとも２行の段落を保持するなどの分割オプションもサポートされています。
    // 余分な作業があるので、このサンプルは完了するまでにかなりの時間がかかります。
    public class LargeDocument3
    {
        public int CreatePDF(Stream stream)
        {
            // 生成するページ数を設定します。
            const int N = Common.Util.LargeDocumentIterations;
            var start = Common.Util.TimeNow();
            var doc = new GcPdfDocument();
            // テキストを保持/書式設定するための TextLayout を準備します。
            var tl = new TextLayout(72)
            {
                MaxWidth = doc.PageSize.Width,
                MaxHeight = doc.PageSize.Height,
                MarginAll = 72,
                FirstLineIndent = 36,
            };
            tl.DefaultFormat.FontName = "Yu Gothic";
            tl.DefaultFormat.FontSize = 12;
            // ドキュメントを生成します。
            for (int paraIdx = 0; paraIdx < N; ++paraIdx)
            {
                tl.AppendLine(Common.Util.getString_ja(1, 0, 1));
            }
            // TextLayout の分割および描画については、PaginatedText サンプルを参照してください。
            var tso = new TextSplitOptions(tl)
            {
                MinLinesInFirstParagraph = 2,
                MinLinesInLastParagraph = 2,
            };
            tl.PerformLayout(true);
            // レイアウトを分割して描画するループ処理。
            var tls = new TextLayoutSplitter(tl);
            for (var tlPage = tls.Split(tso); tlPage != null; tlPage = tls.Split(tso))
                doc.NewPage().Graphics.DrawTextLayout(tlPage, PointF.Empty);

            tl.Clear();
            // タイトルページを挿入します（StartDoc/EndDoc を使用している場合はできません）。
            tl.FirstLineIndent = 0;
            var fnt = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "yumin.ttf"));
            var tf0 = new TextFormat() { FontSize = 24, FontBold = true, Font = fnt };
            tl.Append(string.Format("大規模なドキュメント\n{0} 個の段落\n\n", N), tf0);
            var tf1 = new TextFormat(tf0) { FontSize = 14, FontItalic = true };
            tl.Append(string.Format("生成日時 {0}、作成時間 {1:m\\m\\ s\\s\\ fff\\m\\s}", Common.Util.TimeNow(), Common.Util.TimeNow() - start), tf1);
            tl.TextAlignment = TextAlignment.Center;
            tl.PerformLayout(true);
            doc.Pages.Insert(0).Graphics.DrawTextLayout(tl, PointF.Empty);
            // PDF ドキュメントを保存します。
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
